@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2017-2018 Intel Corporation. All rights reserved.
+ * Copyright(c) 2017-2019 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -55,7 +55,6 @@
  * Local data structures
  * ---------------------------------------
  */
-static const struct pqos_cap *m_cap = NULL;
 static const struct pqos_cpuinfo *m_cpu = NULL;
 
 /**
@@ -63,57 +62,82 @@ static const struct pqos_cpuinfo *m_cpu = NULL;
  *
  * @param l3_cdp_cfg L3 CDP option
  * @param l2_cdp_cfg L2 CDP option
+ * @param mba_cfg requested MBA config
+ *
  * @return Operational status
  * @retval PQOS_RETVAL_OK on success
  */
 static int
 os_interface_mount(const enum pqos_cdp_config l3_cdp_cfg,
-                   const enum pqos_cdp_config l2_cdp_cfg)
+                   const enum pqos_cdp_config l2_cdp_cfg,
+                   const enum pqos_mba_config mba_cfg)
 {
         const struct pqos_cap_l3ca *l3_cap = NULL;
         const struct pqos_cap_l2ca *l2_cap = NULL;
+        const struct pqos_cap_mba *mba_cap = NULL;
         const struct pqos_capability *alloc_cap = NULL;
 
-        if ((l3_cdp_cfg != PQOS_REQUIRE_CDP_ON &&
-            l3_cdp_cfg != PQOS_REQUIRE_CDP_OFF) || (
-            l2_cdp_cfg != PQOS_REQUIRE_CDP_ON &&
-            l2_cdp_cfg != PQOS_REQUIRE_CDP_OFF)) {
-                LOG_ERROR("Invalid CDP mounting setting %d!\n",
-                          l3_cdp_cfg);
+        if (l3_cdp_cfg != PQOS_REQUIRE_CDP_ON &&
+            l3_cdp_cfg != PQOS_REQUIRE_CDP_OFF) {
+                LOG_ERROR("Invalid L3 CDP mounting setting %d!\n", l3_cdp_cfg);
+                return PQOS_RETVAL_PARAM;
+        }
+
+        if (l2_cdp_cfg != PQOS_REQUIRE_CDP_ON &&
+            l2_cdp_cfg != PQOS_REQUIRE_CDP_OFF) {
+                LOG_ERROR("Invalid L2 CDP mounting setting %d!\n", l2_cdp_cfg);
+                return PQOS_RETVAL_PARAM;
+        }
+
+        if (mba_cfg != PQOS_MBA_DEFAULT &&
+            mba_cfg != PQOS_MBA_CTRL) {
+                LOG_ERROR("Invalid MBA mounting setting %d!\n", mba_cfg);
                 return PQOS_RETVAL_PARAM;
         }
 
         if (l3_cdp_cfg == PQOS_REQUIRE_CDP_OFF &&
-                l2_cdp_cfg == PQOS_REQUIRE_CDP_OFF)
+            l2_cdp_cfg == PQOS_REQUIRE_CDP_OFF &&
+            mba_cfg == PQOS_MBA_DEFAULT)
                 goto mount;
 
         /* Get L3 CAT capabilities */
-        (void) pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L3CA, &alloc_cap);
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_L3CA, &alloc_cap);
         if (alloc_cap != NULL)
                 l3_cap = alloc_cap->u.l3ca;
 
         /* Get L2 CAT capabilities */
-        (void) pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L2CA, &alloc_cap);
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_L2CA, &alloc_cap);
         if (alloc_cap != NULL)
                 l2_cap = alloc_cap->u.l2ca;
 
-        if (l3_cap != NULL && !l3_cap->os_cdp &&
+        /* Get MBA capabilities */
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_MBA, &alloc_cap);
+        if (alloc_cap != NULL)
+                mba_cap = alloc_cap->u.mba;
+
+        if (l3_cap != NULL && !l3_cap->cdp &&
                 l3_cdp_cfg == PQOS_REQUIRE_CDP_ON) {
                 /* Check against erroneous CDP request */
                 LOG_ERROR("L3 CDP requested but not supported by the "
                           "platform!\n");
                 return PQOS_RETVAL_PARAM;
         }
-        if (l2_cap != NULL && !l2_cap->os_cdp &&
+        if (l2_cap != NULL && !l2_cap->cdp &&
                 l2_cdp_cfg == PQOS_REQUIRE_CDP_ON) {
                 /* Check against erroneous CDP request */
                 LOG_ERROR("L2 CDP requested but not supported by the "
                           "platform!\n");
                 return PQOS_RETVAL_PARAM;
         }
+        if (mba_cap != NULL && mba_cap->ctrl == 0 &&
+                mba_cfg == PQOS_MBA_CTRL) {
+                /* Check against erroneous MBA request */
+                LOG_ERROR("MBA CTRL requested but not supported!\n");
+                return PQOS_RETVAL_PARAM;
+        }
 
  mount:
-        return resctrl_mount(l3_cdp_cfg, l2_cdp_cfg);
+        return resctrl_mount(l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
 }
 
 /**
@@ -126,43 +150,41 @@ static int
 os_alloc_check(void)
 {
         int ret;
-        unsigned i, supported = 0;
+        const struct pqos_capability *l3_cap;
+        const struct pqos_capability *l2_cap;
+        const struct pqos_capability *mba_cap;
+
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_L3CA, &l3_cap);
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_L2CA, &l2_cap);
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_MBA, &mba_cap);
 
         /**
          * Check if resctrl is supported
          */
-        for (i = 0; i < m_cap->num_cap; i++) {
-                if (m_cap->capabilities[i].os_support == 1) {
-                        if (m_cap->capabilities[i].type == PQOS_CAP_TYPE_L3CA)
-                                supported = 1;
-                        if (m_cap->capabilities[i].type == PQOS_CAP_TYPE_L2CA)
-                                supported = 1;
-                        if (m_cap->capabilities[i].type == PQOS_CAP_TYPE_MBA)
-                                supported = 1;
-                }
-        }
-
-        if (!supported)
+        if (l3_cap == NULL && l2_cap == NULL && mba_cap == NULL)
                 return PQOS_RETVAL_OK;
+
         /**
          * Check if resctrl is mounted
          */
         if (access(RESCTRL_PATH"/cpus", F_OK) != 0) {
-                const struct pqos_capability *alloc_cap = NULL;
                 enum pqos_cdp_config l3_cdp_mount = PQOS_REQUIRE_CDP_OFF;
                 enum pqos_cdp_config l2_cdp_mount = PQOS_REQUIRE_CDP_OFF;
+                enum pqos_mba_config mba_mount = PQOS_MBA_DEFAULT;
 
-                /* Get L3 CAT capabilities */
-                (void) pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L3CA, &alloc_cap);
-                if (alloc_cap != NULL && alloc_cap->u.l3ca->cdp_on)
+                /* Check L3 CAT capabilities */
+                if (l3_cap != NULL && l3_cap->u.l3ca->cdp_on)
                         l3_cdp_mount = PQOS_REQUIRE_CDP_ON;
 
-                /* Get L2 CAT capabilities */
-                (void) pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L2CA, &alloc_cap);
-                if (alloc_cap != NULL && alloc_cap->u.l2ca->cdp_on)
+                /* Check L2 CAT capabilities */
+                if (l2_cap != NULL && l2_cap->u.l2ca->cdp_on)
                         l2_cdp_mount = PQOS_REQUIRE_CDP_ON;
 
-                ret = os_interface_mount(l3_cdp_mount, l2_cdp_mount);
+                /* Check MBA capabilities */
+                if (mba_cap != NULL && mba_cap->u.mba->ctrl_on)
+                        mba_mount = PQOS_MBA_CTRL;
+
+                ret = os_interface_mount(l3_cdp_mount, l2_cdp_mount, mba_mount);
                 if (ret != PQOS_RETVAL_OK) {
                         LOG_INFO("Unable to mount resctrl\n");
                         return PQOS_RETVAL_RESOURCE;
@@ -183,10 +205,12 @@ static int
 os_alloc_prep(void)
 {
         unsigned i, num_grps = 0;
+        const struct pqos_cap *cap;
         int ret;
 
-        ASSERT(m_cap != NULL);
-        ret = resctrl_alloc_get_grps_num(m_cap, &num_grps);
+        _pqos_cap_get(&cap, NULL);
+
+        ret = resctrl_alloc_get_grps_num(cap, &num_grps);
 	if (ret != PQOS_RETVAL_OK)
 		return ret;
         /*
@@ -225,7 +249,6 @@ os_alloc_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
         if (cpu == NULL || cap == NULL)
 		return PQOS_RETVAL_PARAM;
 
-	m_cap = cap;
 	m_cpu = cpu;
 
         ret = os_alloc_check();
@@ -246,7 +269,6 @@ os_alloc_fini(void)
 {
         int ret = PQOS_RETVAL_OK;
 
-        m_cap = NULL;
         m_cpu = NULL;
         return ret;
 }
@@ -258,15 +280,16 @@ os_alloc_assoc_set(const unsigned lcore, const unsigned class_id)
 	unsigned grps;
         int ret_mon;
         char mon_group[256];
+        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu;
 
-        ASSERT(m_cpu != NULL);
-        ASSERT(m_cap != NULL);
+        _pqos_cap_get(&cap, &cpu);
 
-        ret = pqos_cpu_check_core(m_cpu, lcore);
+        ret = pqos_cpu_check_core(cpu, lcore);
         if (ret != PQOS_RETVAL_OK)
                 return PQOS_RETVAL_PARAM;
 
-        ret = resctrl_alloc_get_grps_num(m_cap, &grps);
+        ret = resctrl_alloc_get_grps_num(cap, &grps);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -284,7 +307,7 @@ os_alloc_assoc_set(const unsigned lcore, const unsigned class_id)
          */
         ret_mon = resctrl_mon_assoc_get(lcore, mon_group, sizeof(mon_group));
         if (ret_mon != PQOS_RETVAL_OK && ret_mon != PQOS_RETVAL_RESOURCE)
-                LOG_WARN("Failed to obtain monitoring group assigment for "
+                LOG_WARN("Failed to obtain monitoring group assignment for "
                          "core %u\n", lcore);
 
         ret = resctrl_alloc_assoc_set(lcore, class_id);
@@ -334,10 +357,13 @@ os_pid_get_pid_assoc(const unsigned class_id, unsigned *count)
         unsigned grps;
         int ret;
         unsigned *tasks;
+        const struct pqos_cap *cap;
 
-	ASSERT(m_cap != NULL);
         ASSERT(count != NULL);
-        ret = resctrl_alloc_get_grps_num(m_cap, &grps);
+
+        _pqos_cap_get(&cap, NULL);
+
+        ret = resctrl_alloc_get_grps_num(cap, &grps);
 	if (ret != PQOS_RETVAL_OK)
 		return NULL;
 
@@ -356,62 +382,6 @@ os_pid_get_pid_assoc(const unsigned class_id, unsigned *count)
         return tasks;
 }
 
-/**
- * @brief Gets unused COS
- *
- * The lowest acceptable COS is 1, as 0 is a default one
- *
- * @param [in] hi_class_id highest acceptable COS id
- * @param [out] class_id unused COS
- *
- * @return Operation status
- */
-static int
-get_unused_cos(const unsigned hi_class_id,
-               unsigned *class_id)
-{
-        unsigned used_classes[hi_class_id + 1];
-        unsigned i, cos;
-        int ret;
-
-        if (class_id == NULL)
-                return PQOS_RETVAL_PARAM;
-
-        memset(used_classes, 0, sizeof(used_classes));
-
-        for (i = hi_class_id; i != 0; i--) {
-                struct resctrl_cpumask mask;
-                unsigned j;
-
-                ret = resctrl_alloc_cpumask_read(i, &mask);
-                if (ret != PQOS_RETVAL_OK)
-			return ret;
-
-                for (j = 0; j < sizeof(mask.tab); j++)
-                        if (mask.tab[j] > 0) {
-                                used_classes[i] = 1;
-                                break;
-                        }
-
-                if (used_classes[i] == 1)
-                        continue;
-
-                ret = resctrl_alloc_task_file_check(i, &used_classes[i]);
-                if (ret != PQOS_RETVAL_OK)
-			return ret;
-        }
-
-        /* Find unused COS */
-        for (cos = hi_class_id; cos != 0; cos--) {
-                if (used_classes[cos] == 0) {
-                        *class_id = cos;
-                        return PQOS_RETVAL_OK;
-                }
-        }
-
-        return PQOS_RETVAL_RESOURCE;
-}
-
 int
 os_alloc_assign(const unsigned technology,
                 const unsigned *core_array,
@@ -420,15 +390,17 @@ os_alloc_assign(const unsigned technology,
 {
         unsigned i, num_rctl_grps = 0;
         int ret;
+        const struct pqos_cap *cap;
 
         ASSERT(core_num > 0);
         ASSERT(core_array != NULL);
         ASSERT(class_id != NULL);
-        ASSERT(m_cap != NULL);
         UNUSED_PARAM(technology);
 
+        _pqos_cap_get(&cap, NULL);
+
         /* obtain highest class id for all requested technologies */
-        ret = resctrl_alloc_get_grps_num(m_cap, &num_rctl_grps);
+        ret = resctrl_alloc_get_grps_num(cap, &num_rctl_grps);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -436,7 +408,7 @@ os_alloc_assign(const unsigned technology,
                 return PQOS_RETVAL_ERROR;
 
         /* find an unused class from highest down */
-        ret = get_unused_cos(num_rctl_grps - 1, class_id);
+        ret = resctrl_alloc_get_unused_group(num_rctl_grps, class_id);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -522,27 +494,32 @@ os_alloc_reset_cores(void)
  *
  * @param [in] l3_cap l3 cache capability
  * @param [in] l2_cap l2 cache capability
+ * @param [in] mba_cap mba capability
  *
  * @return Operation status
  */
 static int
 os_alloc_reset_schematas(const struct pqos_cap_l3ca *l3_cap,
-                         const struct pqos_cap_l2ca *l2_cap)
+                         const struct pqos_cap_l2ca *l2_cap,
+                         const struct pqos_cap_mba *mba_cap)
 {
-	const unsigned default_mba = 100;
+	uint32_t default_mba = 100;
 	uint64_t default_l3ca = 0;
 	uint64_t default_l2ca = 0;
 	unsigned grps;
 	unsigned i, j;
 	int ret;
+        const struct pqos_cap *cap;
 
 	LOG_INFO("OS alloc reset - schematas\n");
 
+        _pqos_cap_get(&cap, NULL);
+
 	ret = resctrl_lock_exclusive();
 	if (ret != PQOS_RETVAL_OK)
 		return ret;
 
-	ret = resctrl_alloc_get_grps_num(m_cap, &grps);
+	ret = resctrl_alloc_get_grps_num(cap, &grps);
 	if (ret != PQOS_RETVAL_OK)
 		goto os_alloc_reset_light_schematas_exit;
 
@@ -555,13 +532,16 @@ os_alloc_reset_schematas(const struct pqos_cap_l3ca *l3_cap,
 	if (l2_cap != NULL)
 		default_l2ca = ((uint64_t)1 << l2_cap->num_ways) - 1;
 
+        if (mba_cap != NULL && mba_cap->ctrl_on)
+                default_mba = UINT32_MAX;
+
 	/**
 	 * Reset schematas
 	 */
 	for (i = 0; i < grps; i++) {
 		struct resctrl_alloc_schemata schmt;
 
-		ret = resctrl_alloc_schemata_init(i, m_cap, m_cpu, &schmt);
+		ret = resctrl_alloc_schemata_init(i, cap, m_cpu, &schmt);
 		if (ret != PQOS_RETVAL_OK) {
 			LOG_ERROR("Error on schemata init "
 				  "for resctrl group %u\n", i);
@@ -591,7 +571,7 @@ os_alloc_reset_schematas(const struct pqos_cap_l3ca *l3_cap,
 		}
 
 		for (j = 0; j < schmt.mba_num; j++)
-			schmt.mba[j].mb_rate = default_mba;
+			schmt.mba[j].mb_max = default_mba;
 
 		ret = resctrl_alloc_schemata_write(i, &schmt);
 
@@ -690,14 +670,16 @@ os_alloc_reset_tasks(void)
  * Resets all mba schematas to default value
  * Moves all tasks to default COS
  *
- * @param [in] l3_cap id of default COS
- * @param [in] l2_cap id of default COS
+ * @param [in] l3_cap L3 CAT capability
+ * @param [in] l2_cap L2 CAT capability
+ * @param [in] mba_cap MBA capability
  *
  * @return Operation status
  */
 static int
 os_alloc_reset_light(const struct pqos_cap_l3ca *l3_cap,
-		     const struct pqos_cap_l2ca *l2_cap)
+		     const struct pqos_cap_l2ca *l2_cap,
+                     const struct pqos_cap_mba *mba_cap)
 {
 	int ret = PQOS_RETVAL_OK;
 	int step_result;
@@ -708,7 +690,7 @@ os_alloc_reset_light(const struct pqos_cap_l3ca *l3_cap,
 	if (step_result != PQOS_RETVAL_OK)
 		ret = step_result;
 
-	step_result = os_alloc_reset_schematas(l3_cap, l2_cap);
+	step_result = os_alloc_reset_schematas(l3_cap, l2_cap, mba_cap);
 	if (step_result != PQOS_RETVAL_OK)
 		ret = step_result;
 
@@ -770,32 +752,71 @@ l2_cdp_update(const enum pqos_cdp_config l2_cdp_cfg,
  */
 static int
 l3_cdp_update(const enum pqos_cdp_config l3_cdp_cfg,
-	      const struct pqos_cap_l3ca *l3_cap,
-	      enum pqos_cdp_config *l3_cdp)
+              const struct pqos_cap_l3ca *l3_cap,
+              enum pqos_cdp_config *l3_cdp)
 {
         int ret = 0;
 
-	if (l3_cap == NULL)
-		return 0;
-
-	switch (l3_cdp_cfg) {
-	case PQOS_REQUIRE_CDP_ON:
-		*l3_cdp = l3_cdp_cfg;
+        if (l3_cap == NULL)
+                return 0;
+        switch (l3_cdp_cfg) {
+        case PQOS_REQUIRE_CDP_ON:
+                *l3_cdp = l3_cdp_cfg;
                 ret = !l3_cap->cdp_on;
-		break;
-	case PQOS_REQUIRE_CDP_ANY:
+                break;
+        case PQOS_REQUIRE_CDP_ANY:
                 if (l3_cap->cdp_on)
                         *l3_cdp = PQOS_REQUIRE_CDP_ON;
                 else
                         *l3_cdp = PQOS_REQUIRE_CDP_OFF;
-		break;
-	case PQOS_REQUIRE_CDP_OFF:
-		*l3_cdp = l3_cdp_cfg;
+                break;
+        case PQOS_REQUIRE_CDP_OFF:
+                *l3_cdp = l3_cdp_cfg;
                 ret = l3_cap->cdp_on;
-		break;
-	}
+                break;
+        }
 
-	return ret;
+        return ret;
+}
+
+/**
+ * @brief updates MBA configuration for MBA capability
+ *
+ * @param [in] mba_cfg requsted MBA configuration
+ * @param [in] mba_cap MBA capability
+ * @param [out] mba_ctrl updated MBA config
+ *
+ * @retval > 0 mba_ctrl change
+ * @retval 0 no change
+ */
+static int
+mba_cfg_update(const enum pqos_mba_config mba_cfg,
+                const struct pqos_cap_mba *mba_cap,
+                enum pqos_mba_config *mba_ctrl)
+{
+        int ret = 0;
+
+        if (mba_cap == NULL)
+                return 0;
+
+        switch (mba_cfg) {
+        case PQOS_MBA_CTRL:
+                *mba_ctrl = mba_cfg;
+                ret = (mba_cap->ctrl_on != 1);
+                break;
+        case PQOS_MBA_ANY:
+                if (mba_cap->ctrl_on == 1)
+                        *mba_ctrl = PQOS_MBA_CTRL;
+                else
+                        *mba_ctrl = PQOS_MBA_DEFAULT;
+                break;
+        case PQOS_MBA_DEFAULT:
+                *mba_ctrl = mba_cfg;
+                ret = (mba_cap->ctrl_on != 0);
+                break;
+        }
+
+        return ret;
 }
 
 /**
@@ -806,16 +827,20 @@ l3_cdp_update(const enum pqos_cdp_config l3_cdp_cfg,
  *
  * @param [in] l3_cdp_cfg requested L3 CAT CDP config
  * @param [in] l2_cdp_cfg requested L2 CAT CDP config
+ * @param [in] mba_cfg requested MBA config
  * @param [in] l3_cap l3 capability
  * @param [in] l2_cap l2 capability
+ * @param [in] mba_cap mba capability
  *
  * @return Operation status
  */
 static int
 os_alloc_reset_full(const enum pqos_cdp_config l3_cdp_cfg,
 		    const enum pqos_cdp_config l2_cdp_cfg,
+                    const enum pqos_mba_config mba_cfg,
 	            const struct pqos_cap_l3ca *l3_cap,
-	            const struct pqos_cap_l2ca *l2_cap)
+	            const struct pqos_cap_l2ca *l2_cap,
+                    const struct pqos_cap_mba *mba_cap)
 {
 	int ret;
 
@@ -840,12 +865,13 @@ os_alloc_reset_full(const enum pqos_cdp_config l3_cdp_cfg,
         /**
          * Mount now with CDP option.
          */
-	LOG_INFO("OS alloc reset - mount resctrl with "
-		"l3_cdp %s and l2_cdp %s\n",
-		l3_cdp_cfg ? "on" : "off",
-		l2_cdp_cfg ? "on" : "off");
+        LOG_INFO("OS alloc reset - mount resctrl with "
+                 "l3_cdp %s, l2_cdp %s, mba ctrl %s\n",
+                 l3_cdp_cfg == PQOS_REQUIRE_CDP_ON ? "on" : "off",
+                 l2_cdp_cfg == PQOS_REQUIRE_CDP_ON ? "on" : "off",
+                 mba_cfg == PQOS_MBA_CTRL ? "on" : "off");
 
-        ret = os_interface_mount(l3_cdp_cfg, l2_cdp_cfg);
+        ret = os_interface_mount(l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
         if (ret != PQOS_RETVAL_OK) {
                 LOG_ERROR("Mount OS interface error!\n");
                 goto os_alloc_reset_full_exit;
@@ -854,6 +880,8 @@ os_alloc_reset_full(const enum pqos_cdp_config l3_cdp_cfg,
                 _pqos_cap_l3cdp_change(l3_cdp_cfg);
         if (l2_cap != NULL)
                 _pqos_cap_l2cdp_change(l2_cdp_cfg);
+        if (mba_cap != NULL)
+                _pqos_cap_mba_change(mba_cfg);
         /**
          * Create the COS dir's in resctrl.
          */
@@ -868,15 +896,19 @@ os_alloc_reset_full(const enum pqos_cdp_config l3_cdp_cfg,
 
 int
 os_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
-               const enum pqos_cdp_config l2_cdp_cfg)
+               const enum pqos_cdp_config l2_cdp_cfg,
+               const enum pqos_mba_config mba_cfg)
 {
         const struct pqos_capability *alloc_cap = NULL;
         const struct pqos_cap_l3ca *l3_cap = NULL;
         const struct pqos_cap_l2ca *l2_cap = NULL;
+        const struct pqos_cap_mba *mba_cap = NULL;
         enum pqos_cdp_config l3_cdp = PQOS_REQUIRE_CDP_OFF;
-	enum pqos_cdp_config l2_cdp = PQOS_REQUIRE_CDP_OFF;
-	int l3_cdp_changed = 0;
-	int l2_cdp_changed = 0;
+        enum pqos_cdp_config l2_cdp = PQOS_REQUIRE_CDP_OFF;
+        enum pqos_mba_config mba_ctrl = PQOS_MBA_DEFAULT;
+        int l3_cdp_changed = 0;
+        int l2_cdp_changed = 0;
+        int mba_changed = 0;
         int ret;
 
         ASSERT(l3_cdp_cfg == PQOS_REQUIRE_CDP_ON ||
@@ -887,21 +919,31 @@ os_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
                l2_cdp_cfg == PQOS_REQUIRE_CDP_OFF ||
                l2_cdp_cfg == PQOS_REQUIRE_CDP_ANY);
 
+        ASSERT(mba_cfg == PQOS_MBA_DEFAULT ||
+               mba_cfg == PQOS_MBA_CTRL ||
+               mba_cfg == PQOS_MBA_ANY);
+
         /* Get L3 CAT capabilities */
-        (void) pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L3CA, &alloc_cap);
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_L3CA, &alloc_cap);
         if (alloc_cap != NULL)
                 l3_cap = alloc_cap->u.l3ca;
 
         /* Get L2 CAT capabilities */
         alloc_cap = NULL;
-        (void) pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L2CA, &alloc_cap);
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_L2CA, &alloc_cap);
         if (alloc_cap != NULL)
                 l2_cap = alloc_cap->u.l2ca;
 
+        /* Get MBA capabilities */
+        alloc_cap = NULL;
+        (void) _pqos_cap_get_type(PQOS_CAP_TYPE_MBA, &alloc_cap);
+        if (alloc_cap != NULL)
+                mba_cap = alloc_cap->u.mba;
+
         /* Check if either L2 CAT or L3 CAT is supported */
-        if (l2_cap == NULL && l3_cap == NULL) {
-		LOG_ERROR("L2 CAT/L3 CAT not present!\n");
-		ret = PQOS_RETVAL_RESOURCE; /* no L2/L3 CAT present */
+        if (l2_cap == NULL && l3_cap == NULL && mba_cap == NULL) {
+		LOG_ERROR("L2 CAT/L3 CAT/MBA not present!\n");
+		ret = PQOS_RETVAL_RESOURCE; /* no L2 CAT/L3 CAT/MBA present */
 		goto os_alloc_reset_exit;
         }
         /* Check L3 CDP requested while not present */
@@ -912,7 +954,7 @@ os_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
         }
         /* Check against erroneous CDP request */
         if (l3_cap != NULL && l3_cdp_cfg == PQOS_REQUIRE_CDP_ON &&
-		!l3_cap->os_cdp) {
+		!l3_cap->cdp) {
 		LOG_ERROR("L3 CAT/CDP requested but not supported by the "
                           "platform!\n");
 		ret = PQOS_RETVAL_PARAM;
@@ -926,12 +968,25 @@ os_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
         }
         /* Check against erroneous L2 CDP request */
         if (l2_cap != NULL && l2_cdp_cfg == PQOS_REQUIRE_CDP_ON &&
-                !l2_cap->os_cdp) {
+                !l2_cap->cdp) {
                 LOG_ERROR("L2 CAT/CDP requested but not supported by the "
                           "platform!\n");
                 ret = PQOS_RETVAL_PARAM;
                 goto os_alloc_reset_exit;
         }
+        /* Check MBA CTRL requested while not present */
+        if (mba_cap == NULL && mba_cfg != PQOS_MBA_ANY) {
+                LOG_ERROR("MBA CTRL setting requested but no MBA present!\n");
+                ret = PQOS_RETVAL_RESOURCE;
+                goto os_alloc_reset_exit;
+        }
+        /* Check against erroneous MBA CTRL request */
+        if (mba_cap != NULL && mba_cfg == PQOS_MBA_CTRL &&
+                mba_cap->ctrl == 0) {
+                LOG_ERROR("MBA CTRL requested but not supported!\n");
+                ret = PQOS_RETVAL_PARAM;
+                goto os_alloc_reset_exit;
+        }
 
         /** Check if resctrl is in use */
         ret = resctrl_lock_exclusive();
@@ -941,8 +996,9 @@ os_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
 
         l3_cdp_changed = l3_cdp_update(l3_cdp_cfg, l3_cap, &l3_cdp);
         l2_cdp_changed = l2_cdp_update(l2_cdp_cfg, l2_cap, &l2_cdp);
+        mba_changed = mba_cfg_update(mba_cfg, mba_cap, &mba_ctrl);
 
-        if (l3_cdp_changed || l2_cdp_changed) {
+        if (l3_cdp_changed || l2_cdp_changed || mba_changed) {
 
 		unsigned monitoring_active = 0;
 
@@ -955,13 +1011,14 @@ os_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
 
 		if (monitoring_active > 0) {
 			LOG_ERROR("Failed to reset allocation. Please stop "
-				  "monitoring in order to change CDP settings\n");
+				  "monitoring in order to change CDP/MBA "
+                                  "settings\n");
 			ret = PQOS_RETVAL_ERROR;
 		} else
-			ret = os_alloc_reset_full(l3_cdp, l2_cdp, l3_cap,
-                                                  l2_cap);
+			ret = os_alloc_reset_full(l3_cdp, l2_cdp, mba_ctrl,
+                                                  l3_cap, l2_cap, mba_cap);
         } else
-		ret = os_alloc_reset_light(l3_cap, l2_cap);
+		ret = os_alloc_reset_light(l3_cap, l2_cap, mba_cap);
 
  os_alloc_reset_exit:
         return ret;
@@ -978,17 +1035,19 @@ os_l3ca_set(const unsigned socket,
 	unsigned i;
 	unsigned num_grps = 0, l3ca_num;
 	int cdp_enabled = 0;
+        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu;
 
 	ASSERT(ca != NULL);
 	ASSERT(num_cos != 0);
-	ASSERT(m_cap != NULL);
-	ASSERT(m_cpu != NULL);
 
-	ret = pqos_l3ca_get_cos_num(m_cap, &l3ca_num);
+        _pqos_cap_get(&cap, &cpu);
+
+	ret = pqos_l3ca_get_cos_num(cap, &l3ca_num);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* L3 CAT not supported */
 
-	ret = resctrl_alloc_get_grps_num(m_cap, &num_grps);
+	ret = resctrl_alloc_get_grps_num(cap, &num_grps);
 	if (ret != PQOS_RETVAL_OK)
 		return ret;
 
@@ -996,7 +1055,7 @@ os_l3ca_set(const unsigned socket,
 		return PQOS_RETVAL_ERROR;
 
 	/* Get number of sockets in the system */
-	sockets = pqos_cpu_get_sockets(m_cpu, &sockets_num);
+	sockets = pqos_cpu_get_sockets(cpu, &sockets_num);
 	if (sockets == NULL || sockets_num == 0) {
 		ret = PQOS_RETVAL_ERROR;
 		goto os_l3ca_set_exit;
@@ -1007,7 +1066,7 @@ os_l3ca_set(const unsigned socket,
 		goto os_l3ca_set_exit;
 	}
 
-	ret = pqos_l3ca_cdp_enabled(m_cap, NULL, &cdp_enabled);
+	ret = pqos_l3ca_cdp_enabled(cap, NULL, &cdp_enabled);
 	if (ret != PQOS_RETVAL_OK)
 		goto os_l3ca_set_exit;
 
@@ -1025,7 +1084,7 @@ os_l3ca_set(const unsigned socket,
 			goto os_l3ca_set_unlock;
 		}
 
-		ret = resctrl_alloc_schemata_init(ca[i].class_id, m_cap, m_cpu,
+		ret = resctrl_alloc_schemata_init(ca[i].class_id, cap, cpu,
 		                                  &schmt);
 
 		/* read schemata file */
@@ -1075,25 +1134,27 @@ os_l3ca_get(const unsigned socket,
 	unsigned count = 0;
 	unsigned sockets_num = 0;
 	unsigned *sockets = NULL;
+        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu;
 
 	ASSERT(num_ca != NULL);
 	ASSERT(ca != NULL);
 	ASSERT(max_num_ca != 0);
-	ASSERT(m_cap != NULL);
-	ASSERT(m_cpu != NULL);
 
-	ret = pqos_l3ca_get_cos_num(m_cap, &count);
+        _pqos_cap_get(&cap, &cpu);
+
+	ret = pqos_l3ca_get_cos_num(cap, &count);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* L3 CAT not supported */
 
-	ret = resctrl_alloc_get_grps_num(m_cap, &count);
+	ret = resctrl_alloc_get_grps_num(cap, &count);
 	if (ret != PQOS_RETVAL_OK)
 		return ret;
 
 	if (count > max_num_ca)
 		return PQOS_RETVAL_ERROR;
 
-	sockets = pqos_cpu_get_sockets(m_cpu, &sockets_num);
+	sockets = pqos_cpu_get_sockets(cpu, &sockets_num);
 	if (sockets == NULL || sockets_num == 0) {
 		ret = PQOS_RETVAL_ERROR;
 		goto os_l3ca_get_exit;
@@ -1111,8 +1172,7 @@ os_l3ca_get(const unsigned socket,
 	for (class_id = 0; class_id < count; class_id++) {
 		struct resctrl_alloc_schemata schmt;
 
-		ret = resctrl_alloc_schemata_init(class_id, m_cap, m_cpu,
-		                                  &schmt);
+		ret = resctrl_alloc_schemata_init(class_id, cap, cpu, &schmt);
 		if (ret == PQOS_RETVAL_OK)
 			ret = resctrl_alloc_schemata_read(class_id, &schmt);
 
@@ -1144,13 +1204,12 @@ os_l3ca_get_min_cbm_bits(unsigned *min_cbm_bits)
 	const struct pqos_capability *l3_cap = NULL;
 	FILE *fd;
 
-	ASSERT(m_cap != NULL);
 	ASSERT(min_cbm_bits != NULL);
 
 	/**
 	 * Get L3 CAT capabilities
 	 */
-	ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L3CA, &l3_cap);
+	ret = _pqos_cap_get_type(PQOS_CAP_TYPE_L3CA, &l3_cap);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* L3 CAT not supported */
 
@@ -1180,23 +1239,26 @@ os_l2ca_set(const unsigned l2id,
 	unsigned *l2ids = NULL;
 	unsigned num_grps = 0, l2ca_num;
         int cdp_enabled;
+        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu;
 
-	ASSERT(m_cap != NULL);
 	ASSERT(ca != NULL);
 	ASSERT(num_cos != 0);
 
-	ret = pqos_l2ca_get_cos_num(m_cap, &l2ca_num);
+        _pqos_cap_get(&cap, &cpu);
+
+	ret = pqos_l2ca_get_cos_num(cap, &l2ca_num);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* L2 CAT not supported */
 
-	ret = resctrl_alloc_get_grps_num(m_cap, &num_grps);
+	ret = resctrl_alloc_get_grps_num(cap, &num_grps);
 	if (ret != PQOS_RETVAL_OK)
 		return ret;
 
 	if (num_cos > num_grps)
 		return PQOS_RETVAL_PARAM;
 
-        ret = pqos_l2ca_cdp_enabled(m_cap, NULL, &cdp_enabled);
+        ret = pqos_l2ca_cdp_enabled(cap, NULL, &cdp_enabled);
         if (ret != PQOS_RETVAL_OK)
                 goto os_l2ca_set_exit;
 
@@ -1212,7 +1274,7 @@ os_l2ca_set(const unsigned l2id,
 	}
 
 	/* Get number of L2 ids in the system */
-	l2ids = pqos_cpu_get_l2ids(m_cpu, &l2ids_num);
+	l2ids = pqos_cpu_get_l2ids(cpu, &l2ids_num);
 	if (l2ids == NULL || l2ids_num == 0) {
 		ret = PQOS_RETVAL_ERROR;
 		goto os_l2ca_set_exit;
@@ -1237,7 +1299,7 @@ os_l2ca_set(const unsigned l2id,
                         goto os_l2ca_set_unlock;
                 }
 
-		ret = resctrl_alloc_schemata_init(ca[i].class_id, m_cap, m_cpu,
+		ret = resctrl_alloc_schemata_init(ca[i].class_id, cap, cpu,
 		                                  &schmt);
 
 		/* read schemata file */
@@ -1287,18 +1349,20 @@ os_l2ca_get(const unsigned l2id,
 	unsigned count = 0;
 	unsigned l2ids_num = 0;
 	unsigned *l2ids = NULL;
+        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu;
 
 	ASSERT(num_ca != NULL);
 	ASSERT(ca != NULL);
 	ASSERT(max_num_ca != 0);
-	ASSERT(m_cap != NULL);
-	ASSERT(m_cpu != NULL);
 
-	ret = pqos_l2ca_get_cos_num(m_cap, &count);
+        _pqos_cap_get(&cap, &cpu);
+
+	ret = pqos_l2ca_get_cos_num(cap, &count);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* L2 CAT not supported */
 
-	ret = resctrl_alloc_get_grps_num(m_cap, &count);
+	ret = resctrl_alloc_get_grps_num(cap, &count);
 	if (ret != PQOS_RETVAL_OK)
 		return ret;
 
@@ -1325,8 +1389,7 @@ os_l2ca_get(const unsigned l2id,
 	for (class_id = 0; class_id < count; class_id++) {
 		struct resctrl_alloc_schemata schmt;
 
-		ret = resctrl_alloc_schemata_init(class_id, m_cap, m_cpu,
-		                                  &schmt);
+		ret = resctrl_alloc_schemata_init(class_id, cap, cpu, &schmt);
 		if (ret == PQOS_RETVAL_OK)
 			ret = resctrl_alloc_schemata_read(class_id, &schmt);
 
@@ -1358,13 +1421,12 @@ os_l2ca_get_min_cbm_bits(unsigned *min_cbm_bits)
 	const struct pqos_capability *l2_cap = NULL;
 	FILE *fd;
 
-	ASSERT(m_cap != NULL);
 	ASSERT(min_cbm_bits != NULL);
 
 	/**
 	 * Get L2 CAT capabilities
 	 */
-	ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_L2CA, &l2_cap);
+	ret = _pqos_cap_get_type(PQOS_CAP_TYPE_L2CA, &l2_cap);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* L2 CAT not supported */
 
@@ -1394,21 +1456,23 @@ os_mba_set(const unsigned socket,
 	unsigned *sockets = NULL;
 	unsigned i, step = 0;
 	unsigned num_grps = 0;
+        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu;
 	const struct pqos_capability *mba_cap = NULL;
 
 	ASSERT(requested != NULL);
 	ASSERT(num_cos != 0);
-	ASSERT(m_cap != NULL);
-	ASSERT(m_cpu != NULL);
+
+        _pqos_cap_get(&cap, &cpu);
 
 	/**
 	 * Check if MBA is supported
 	 */
-	ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_MBA, &mba_cap);
+	ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MBA, &mba_cap);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* MBA not supported */
 
-	ret = resctrl_alloc_get_grps_num(m_cap, &num_grps);
+	ret = resctrl_alloc_get_grps_num(cap, &num_grps);
 	if (ret != PQOS_RETVAL_OK)
 		return ret;
 
@@ -1431,18 +1495,32 @@ os_mba_set(const unsigned socket,
 	sockets = pqos_cpu_get_sockets(m_cpu, &sockets_num);
 	if (sockets == NULL || sockets_num == 0 || socket >= sockets_num) {
 		ret = PQOS_RETVAL_ERROR;
-		goto os_l3ca_set_exit;
+		goto os_mba_set_exit;
 	}
 
         ret = resctrl_lock_exclusive();
         if (ret != PQOS_RETVAL_OK)
-                goto os_l3ca_set_exit;
+                goto os_mba_set_exit;
 
 	for (i = 0; i < num_cos; i++) {
 		struct resctrl_alloc_schemata schmt;
 
+		if (!mba_cap->u.mba->ctrl_on && requested[i].ctrl) {
+			LOG_ERROR("MBA controller requested but"
+			          " not enabled!\n");
+			ret = PQOS_RETVAL_PARAM;
+			goto os_mba_set_unlock;
+		}
+
+		if (mba_cap->u.mba->ctrl_on && !requested[i].ctrl) {
+			LOG_ERROR("Expected MBA controller but"
+			          " not requested!\n");
+			ret = PQOS_RETVAL_PARAM;
+			goto os_mba_set_unlock;
+		}
+
 		ret = resctrl_alloc_schemata_init(requested[i].class_id,
-		                                  m_cap, m_cpu, &schmt);
+		                                  cap, cpu, &schmt);
 
 		/* read schemata file */
 		if (ret == PQOS_RETVAL_OK)
@@ -1454,13 +1532,16 @@ os_mba_set(const unsigned socket,
 			struct pqos_mba *mba = &(schmt.mba[socket]);
 
 			*mba = requested[i];
-                        mba->mb_rate = (((requested[i].mb_rate
-                                          + (step/2)) / step) * step);
-			if (mba->mb_rate == 0)
-				mba->mb_rate = step;
 
-			ret = resctrl_alloc_schemata_write(
-				requested[i].class_id, &schmt);
+			if (mba->ctrl == 0) {
+                                mba->mb_max = (((requested[i].mb_max
+                                                  + (step/2)) / step) * step);
+                                if (mba->mb_max == 0)
+                                        mba->mb_max = step;
+			}
+
+                        ret = resctrl_alloc_schemata_write(
+                                requested[i].class_id, &schmt);
 		}
 
 		if (actual != NULL) {
@@ -1476,13 +1557,13 @@ os_mba_set(const unsigned socket,
 		resctrl_alloc_schemata_fini(&schmt);
 
 		if (ret != PQOS_RETVAL_OK)
-			goto os_l3ca_set_unlock;
+			goto os_mba_set_unlock;
 	}
 
- os_l3ca_set_unlock:
+ os_mba_set_unlock:
         resctrl_lock_release();
 
- os_l3ca_set_exit:
+ os_mba_set_exit:
 	if (sockets != NULL)
 		free(sockets);
 
@@ -1500,29 +1581,31 @@ os_mba_get(const unsigned socket,
 	unsigned count = 0;
 	unsigned sockets_num = 0;
 	unsigned *sockets = NULL;
+        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu;
 	const struct pqos_capability *mba_cap = NULL;
 
 	ASSERT(num_cos != NULL);
 	ASSERT(max_num_cos != 0);
 	ASSERT(mba_tab != NULL);
-	ASSERT(m_cap != NULL);
-	ASSERT(m_cpu != NULL);
+
+        _pqos_cap_get(&cap, &cpu);
 
 	/**
 	 * Check if MBA is supported
 	 */
-	ret = pqos_cap_get_type(m_cap, PQOS_CAP_TYPE_MBA, &mba_cap);
+	ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MBA, &mba_cap);
 	if (ret != PQOS_RETVAL_OK)
 		return PQOS_RETVAL_RESOURCE; /* MBA not supported */
 
-	ret = resctrl_alloc_get_grps_num(m_cap, &count);
+	ret = resctrl_alloc_get_grps_num(cap, &count);
 	if (ret != PQOS_RETVAL_OK)
 		return ret;
 
 	if (count > max_num_cos)
 		return PQOS_RETVAL_ERROR;
 
-	sockets = pqos_cpu_get_sockets(m_cpu, &sockets_num);
+	sockets = pqos_cpu_get_sockets(cpu, &sockets_num);
 	if (sockets == NULL || sockets_num == 0 || socket >= sockets_num) {
 		ret = PQOS_RETVAL_ERROR;
 		goto os_mba_get_exit;
@@ -1535,8 +1618,7 @@ os_mba_get(const unsigned socket,
 	for (class_id = 0; class_id < count; class_id++) {
 		struct resctrl_alloc_schemata schmt;
 
-		ret = resctrl_alloc_schemata_init(class_id, m_cap, m_cpu,
-		                                  &schmt);
+		ret = resctrl_alloc_schemata_init(class_id, cap, cpu, &schmt);
 		if (ret == PQOS_RETVAL_OK)
 			ret = resctrl_alloc_schemata_read(class_id, &schmt);
 
@@ -1568,11 +1650,12 @@ os_alloc_assoc_set_pid(const pid_t task,
 	unsigned max_cos = 0;
         int ret_mon;
         char mon_group[256];
+        const struct pqos_cap *cap;
 
-        ASSERT(m_cap != NULL);
+        _pqos_cap_get(&cap, NULL);
 
 	/* Get number of COS */
-        ret = resctrl_alloc_get_grps_num(m_cap, &max_cos);
+        ret = resctrl_alloc_get_grps_num(cap, &max_cos);
 	if (ret != PQOS_RETVAL_OK)
 		return ret;
 
@@ -1591,7 +1674,7 @@ os_alloc_assoc_set_pid(const pid_t task,
          */
         ret_mon = resctrl_mon_assoc_get_pid(task, mon_group, sizeof(mon_group));
         if (ret_mon != PQOS_RETVAL_OK && ret_mon != PQOS_RETVAL_RESOURCE)
-                LOG_WARN("Failed to obtain monitoring group assigment for "
+                LOG_WARN("Failed to obtain monitoring group assignment for "
                          "task %d\n", task);
 
         /* Write to tasks file */
@@ -1641,15 +1724,18 @@ os_alloc_assign_pid(const unsigned technology,
 {
         unsigned i, num_rctl_grps = 0;
         int ret;
+        const struct pqos_cap *cap;
+
 
         ASSERT(task_num > 0);
         ASSERT(task_array != NULL);
         ASSERT(class_id != NULL);
-        ASSERT(m_cap != NULL);
         UNUSED_PARAM(technology);
 
+        _pqos_cap_get(&cap, NULL);
+
         /* obtain highest class id for all requested technologies */
-        ret = resctrl_alloc_get_grps_num(m_cap, &num_rctl_grps);
+        ret = resctrl_alloc_get_grps_num(cap, &num_rctl_grps);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
@@ -1661,7 +1747,7 @@ os_alloc_assign_pid(const unsigned technology,
                 return ret;
 
         /* find an unused class from highest down */
-        ret = get_unused_cos(num_rctl_grps - 1, class_id);
+        ret = resctrl_alloc_get_unused_group(num_rctl_grps, class_id);
         if (ret != PQOS_RETVAL_OK)
                 goto os_alloc_assign_pid_unlock;
 
