@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2018 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2019 Intel Corporation. All rights reserved.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -57,7 +57,7 @@ extern "C" {
  * =======================================
  */
 
-#define PQOS_VERSION       20100        /**< version 2.1.0 */
+#define PQOS_VERSION       30000        /**< version 3.0.0 */
 #define PQOS_MAX_L3CA_COS  16           /**< 16 x COS */
 #define PQOS_MAX_L2CA_COS  16           /**< 16 x COS */
 
@@ -189,11 +189,9 @@ struct pqos_cap_l3ca {
         unsigned way_size;              /**< way size in bytes */
         uint64_t way_contention;        /**< ways contention bit mask */
         int cdp;                        /**< code data prioritization feature
-                                           presence */
+                                           support */
         int cdp_on;                     /**< code data prioritization on or
                                            off */
-        int os_cdp;                     /**< flag to show if CDP is supported
-                                        by OS interface */
 };
 
 /**
@@ -206,11 +204,20 @@ struct pqos_cap_l2ca {
         unsigned way_size;              /**< way size in bytes */
         uint64_t way_contention;        /**< ways contention bit mask */
         int cdp;                        /**< code data prioritization feature
-                                           presence */
+                                           support */
         int cdp_on;                     /**< code data prioritization on or
                                            off */
-        int os_cdp;                     /**< flag to show if CDP is supported
-                                           by OS interface */
+};
+
+/**
+ * Memory Bandwidth Allocation configuration enumeration
+ */
+enum pqos_mba_config {
+        PQOS_MBA_ANY,                   /**< currently enabled configuration */
+        PQOS_MBA_DEFAULT,               /**< direct MBA hardware
+                                           configuration (percentage) */
+        PQOS_MBA_CTRL                   /**< MBA controller configuration
+                                           (MBps) */
 };
 
 /**
@@ -222,6 +229,8 @@ struct pqos_cap_mba {
         unsigned throttle_max;          /**< the max MBA can be throttled */
         unsigned throttle_step;         /**< MBA granularity */
         int is_linear;                  /**< the type of MBA linear/nonlinear */
+        int ctrl;                       /**< MBA controller support */
+        int ctrl_on;                    /**< MBA controller enabled */
 };
 
 /**
@@ -240,15 +249,6 @@ enum pqos_mon_event {
         PQOS_PERF_EVENT_IPC    = 0x8000, /**< instructions per clock */
 };
 
-/**
- * OS monitoring status
- */
-enum pqos_os_mon {
-        PQOS_OS_MON_UNSUPPORTED = 0,    /**< OS monitoring is not supported */
-        PQOS_OS_MON_PERF        = 1,    /**< Perf monitoring is supported */
-        PQOS_OS_MON_RESCTRL     = 2     /**< Resctrl monitoring is supported */
-};
-
 /**
  * Monitoring capabilities structure
  *
@@ -266,8 +266,6 @@ struct pqos_monitor {
                                            this event */
         uint32_t scale_factor;          /**< factor to scale RMID value
                                            to bytes */
-        enum pqos_os_mon os_support;    /**< flag to show if OS monitoring
-                                           supported */
 };
 
 struct pqos_cap_mon {
@@ -284,7 +282,6 @@ struct pqos_cap_mon {
  */
 struct pqos_capability {
         enum pqos_cap_type type;
-        int os_support;                 /**< OS support presence */
         union {
                 struct pqos_cap_mon *mon;
                 struct pqos_cap_l3ca *l3ca;
@@ -723,17 +720,19 @@ int pqos_alloc_release_pid(const pid_t *task_array,
  * - all COS are set to give access to entire resource
  *
  * As part of allocation reset CDP reconfiguration can be performed.
- * This can be requested via \a l3_cdp_cfg and \a l2_cdp_cfg.
+ * This can be requested via \a l3_cdp_cfg, \a l2_cdp_cfg and \a mba_cfg.
  *
  * @param [in] l3_cdp_cfg requested L3 CAT CDP config
  * @param [in] l2_cdp_cfg requested L2 CAT CDP config
+ * @param [in] mba_cfg requested MBA config
  *
  * @return Operation status
  * @retval PQOS_RETVAL_OK on success
  */
 int
 pqos_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
-                 const enum pqos_cdp_config l2_cdp_cfg);
+                 const enum pqos_cdp_config l2_cdp_cfg,
+                 const enum pqos_mba_config mba_cfg);
 
 /*
  * =======================================
@@ -875,7 +874,11 @@ int pqos_l2ca_get_min_cbm_bits(unsigned *min_cbm_bits);
  */
 struct pqos_mba {
         unsigned class_id;      /**< class of service */
-        unsigned mb_rate;       /**< valve open rate (VOR) in percentage */
+        unsigned mb_max;        /**< maximum available bandwidth in percentage
+                                   (without MBA controller) or in MBps
+                                   (with MBA controller), depending on ctrl
+                                   flag */
+        int ctrl;               /**< MBA controller flag */
 };
 
 /**
@@ -1181,6 +1184,18 @@ pqos_l2ca_cdp_enabled(const struct pqos_cap *cap,
                       int *cdp_supported,
                       int *cdp_enabled);
 
+/**
+ * @brief Retrieves MBA controller configuration status
+ *
+ * @param [in] cap platform QoS capabilities structure
+ *                 returned by \a pqos_cap_get
+ * @param [out] ctrl_supported place to store MBA controller support status
+ * @param [out] ctrl_enabled place to store MBA controller enable status
+ *
+ */
+int pqos_mba_ctrl_enabled(const struct pqos_cap *cap, int *ctrl_supported,
+                          int *ctrl_enabled);
+
 /**
  * @brief Retrieves a monitoring value from a group for a specific event.
  * @param [out] value monitoring value
